//===================================CISCKER====================================
//====================================ALU=======================================
//------------------------------------------------------------------------------
//File Name:	ALU.v
//Purpose:		Arithmetic Logic Unit and shift operations
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module ALU(
	//control
	input [2:0]alu_src_a,
	input [1:0]alu_src_b,
	input [5:0]alu_op,
	//in
	input [15:0]ACCA,
	input [7:0]ACCB,
	input [15:0]ACCD,
	input [15:0]MM,
	input [15:0]PC,
	input [15:0]SP,
	input [15:0]IX,
	input [7:0] hidden,
	//out
	output [2:0]ccr_data_a,
	output [1:0]ccr_data_b,
	output reg[15:0]alu_result
	);
	
/////////////////////////////////////////////////////////////////////
//Control Signals
/////////////////////////////////////////////////////////////////////

wire a_inv 				= alu_op[5];
wire b_inv 				= alu_op[4];
wire carry_in			= alu_op[3];
wire [2:0]operation 	= alu_op[2:0];

/////////////////////////////////////////////////////////////////////
//ALU Source A
/////////////////////////////////////////////////////////////////////
reg [15:0] src_a;

always @ *
	case (alu_src_a)
	3'b000:		src_a <= {ACCA};
	3'b001:		src_a <= {8'h00,ACCB};
	3'b010:		src_a <= ACCD;
	3'b011:		src_a <= MM;
	3'b100:		src_a <= PC;
	3'b101:		src_a <= SP;
	3'b110:		src_a <= IX;
	3'b111:		src_a <= 0;
	endcase

/////////////////////////////////////////////////////////////////////
//ALU Source B
/////////////////////////////////////////////////////////////////////
reg [15:0] src_b;

always @ *
	case (alu_src_b)
	2'b00:		src_b <= 0;
	2'b01:		src_b <= {8'h00,ACCB};
	2'b10:		src_b <= hidden;
	2'b11:		src_b <= MM;
	endcase
	
/////////////////////////////////////////////////////////////////////
//Shifter
/////////////////////////////////////////////////////////////////////

wire [8:0] logic_shift;
wire [8:0] arith_shift;
wire [8:0] rotate;

wire left_right = alu_op[4]; //Temp

shifter shifter_inst (
		//in
		.data_in(src_a[7:0]),
		.left_right(left_right),
		.carry(carry_in), 
		//out
		.logic_shift(logic_shift),
		.arith_shift(arith_shift),
		.rotate(rotate)
		);

/////////////////////////////////////////////////////////////////////
//Operation
/////////////////////////////////////////////////////////////////////
	
wire [15:0] a = a_inv ? ~src_a : src_a;
wire [15:0] b = b_inv ? ~src_b : src_b;


always @ *
	case (operation)
	3'b000:		alu_result <= a + b + carry_in;
	3'b001:		alu_result <= a & b;
	3'b010:		alu_result <= a | b;
	3'b011:		alu_result <= a ^ b;
	3'b100:		alu_result <= {7'h00,logic_shift};
	3'b101:		alu_result <= {7'h00,arith_shift};
	3'b110:		alu_result <= {7'h00,rotate};
	3'b111:		alu_result <= src_a;
	endcase
	
/////////////////////////////////////////////////////////////////////
//CCR data
/////////////////////////////////////////////////////////////////////
assign ccr_data_a = {src_a[15],src_a[7],src_a[0]};
assign ccr_data_b = {src_b[15],src_b[7]};

endmodule
	