//===================================CISCKER====================================
//------------------------------------------------------------------------------
//File Name:	CISCKER_Observer.v
//Purpose:		Instantiation of CISCKER processor and state signals connected to the UART module
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------

`timescale 1ns / 1ps

module CISCKER_Observer 
	(
	input CLK_50M,
	input [1:0]PButton,
	input [3:0]SW,
	output [7:0]port,
	input Rx,
	output Tx
	);

//====================Configuration Byte==============
wire [7:0]conf_byte;
reg [25:0] clk_divider;

always @ (*)
case (conf_byte[2:0])
	0:		clk_divider	=	50000000;	//0.5Hz
	1:		clk_divider	=	25000000;	//1Hz
	2:		clk_divider	=	12500000;	//2Hz
	3:		clk_divider =	5000000;		//5Hz
	4:		clk_divider =	2500000;		//10Hz
	5:		clk_divider =	1250000;		//20Hz
	6:		clk_divider =	250000;		//100Hz
	7:		clk_divider =	25000;		//1kHz
endcase

//================Clock Frequency====================
wire div_clk;
wire Rx_Last_Byte;
wire clk_reset = Rx_Last_Byte|conf_byte[3];

Generate_Arbitrary_Divided_Clk32 clk_div(
	.inclk(CLK_50M),
	.outclk(div_clk),
	.div_clk_count(clk_divider),
	.reset(clk_reset)
	);

//================Clocks Phase Shift=================

//With this signal the processors clocks can be stopped for stand-by functions.
wire stop = 0;
	
reg clk_rst = 1;
always @ (posedge div_clk)
	clk_rst <= stop;

phaser phased_clks (
	.clk_in(div_clk),
	.reset(clk_rst),
	.clk_270(clk_270),
	.clk_180(clk_180),
	.clk_90(clk_90),
	.clk_0(clk_0)
    );

//===============Initial State and Reset=============

reg load = 1;
always @ (posedge clk_90)
	if(PButton[0])
		load <= 0;
	else
		load <= 1;

//======================Processor====================
wire [15:0] input_port;
wire [175:0] observer_state;

datapath datapath_ciscker (
    .clk_0(clk_0), 
    .clk_90(clk_90), 
    .clk_180(clk_180), 
    .clk_270(clk_270), 
    .load(load),
	.input_port1(input_port[7:0]),
	.input_port2(input_port[15:8]),
	.observer_state(observer_state)
    );

//================External Assignments===============

assign port = observer_state[159:152];

//====================UART-Observer==================


	UART_top UART_inst(
	.clk(~CLK_50M),
	.DataOut(observer_state),
	.Tx(Tx),
	.Rx(Rx),
	.DataIn({conf_byte,input_port}),
	.Trigger(~div_clk),
	.Rx_Last_Byte(Rx_Last_Byte)
	);
	
	defparam UART_inst.RxBytes = 3;		//	{ConfByte[7:0],InputPort[15:0]}
										//	4	2	2	1	2	2	2	2	1	2	2
	defparam UART_inst.TxBytes = 22;	//	TMR1,2 OP SP CCR IX ACD ALU MM OpCode MAR PC
	defparam UART_inst.baud_rate = 115200;


endmodule 