//===================================CISCKER====================================
//===========================Interruption Controller============================
//------------------------------------------------------------------------------
//File Name:	interrupt.v
//Purpose:		Manage all interruptions and return the address of the ISR
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------

module interrupt(	
										//
										input swi, //Interruption State Flag
										input rti, //Reset Interruption 
										input I_mask, //Interruption Mask
										//PORT IN
										input int_ext1, //External Interruption
										input int_ext2, //External Interruption
										//TIMERS
										input int_timer1,
										input int_timer2,
										//REG_FILE
										input [7:0] int_ctrl, //Contro Flags Register
										//OSC
										input clk,
										//OUTPUTS
										output reg interruption=0,	//Interruption Flag
										output [15:0]int_address	//Interruption Vector Address
				);

//////////////////////////////////////////////////////////////////////////
//Control Flags Register
//////////////////////////////////////////////////////////////////////////
wire reset_flag_int_timer1	=int_ctrl[0]; 
wire reset_flag_int_timer2	=int_ctrl[1];
wire reset_flag_int_ext1	=int_ctrl[2];
wire reset_flag_int_ext2	=int_ctrl[3];
wire enable_int_tr1			=int_ctrl[4];
wire enable_int_tr2			=int_ctrl[5];
wire enable_int_ext1		=int_ctrl[6];
wire enable_int_ext2		=int_ctrl[7];

//////////////////////////////////////////////////////////////////////////
//Interruption Flags Register
//////////////////////////////////////////////////////////////////////////
reg flag_int_t2=0, 		//Timer2 Interruption
	 flag_int_t1=0, 	//Timer1 Interruption
	 flag_int_ext1=0,	//External Interruption 1
	 flag_int_ext2=0;	//External Interruption 2
	 
//////////////////////////////////////////////////////////////////////////	  
//Timer Interruptions
//////////////////////////////////////////////////////////////////////////

always@ (posedge int_timer2, posedge reset_flag_int_timer2)
begin
		if (reset_flag_int_timer2)
			flag_int_t2<=1'b0;
		else
				if (enable_int_tr2)
				flag_int_t2<=1'b1;
				else
				flag_int_t2<=flag_int_t2;
end


always@(posedge int_timer1, posedge reset_flag_int_timer1)
begin
		if (reset_flag_int_timer1)
			flag_int_t1<=1'b0;
		else
				if (enable_int_tr1)
				flag_int_t1<=1'b1;
				else
				flag_int_t1<=flag_int_t1;
end

//////////////////////////////////////////////////////////////////////////
//External Interruptions
//////////////////////////////////////////////////////////////////////////

always@(posedge int_ext1, posedge reset_flag_int_ext1)
begin
		if (reset_flag_int_ext1)
			flag_int_ext1<=1'b0;
		else	
			if (enable_int_ext1)
				flag_int_ext1<=1'b1;
			else
				flag_int_ext1<=flag_int_ext1;		
end

always@(posedge int_ext2, posedge reset_flag_int_ext2)
begin
		if (reset_flag_int_ext2)
			flag_int_ext2<=1'b0;
		else	
				if (enable_int_ext2)
					flag_int_ext2<=1'b1;
				else
					flag_int_ext2<=flag_int_ext2;
end
//////////////////////////////////////////////////////////////////////////
//Interruption State	 
//////////////////////////////////////////////////////////////////////////

//Global Interruption Enable "Toggle" Register
reg global_en_int = 0;

always@(posedge swi, posedge rti)
if(rti)
	global_en_int<=0;
else
	global_en_int<=1'b1;

//Interruption State Register
reg [3:0] int_state=0;  

always @ (posedge clk)
if (global_en_int)
	int_state <= 0;
else
	int_state <= {flag_int_ext2,flag_int_ext1,flag_int_t2,flag_int_t1};

wire or_interrupt = |int_state;

always@(posedge or_interrupt, posedge I_mask)
	if(I_mask)	
		interruption<=1'b0;
	else
		interruption<=1'b1;
		
// Interruption Service Routine Addresses
localparam addr_swi		= 16'h7ff6-1;
localparam addr_int_ext1= 16'h7ff8-1;
localparam addr_int_ext2= 16'h7ffa-1;
localparam addr_int_t1	= 16'h7ffc-1;
localparam addr_int_t2	= 16'h7ffe-1;

assign int_address = flag_int_ext1 	? addr_int_ext1 : //Higher Priority
							flag_int_ext2 	? addr_int_ext2 : 
							flag_int_t1		? addr_int_t1 :
							flag_int_t2		? addr_int_t2 :	//Lower Priority
							addr_swi;	//default
							
endmodule
